-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- Counter_mod12 prototype in LAB7 for adapting the application to a DE10-Lite
-- CLK_Generator chip
-- Strategy: Schematic using frequency dividers and T_FF to square signals
---------------------------------------------------
-- Project P8. Lecture L8.2 on designing CLK_generators  
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html

-- Translation Fig. 17 schematic in LAB7 at: 
-- https://digsys.upc.edu/csd/P07/lab07/lab7.html#prototyping
--------------------------------------------------------------------------------
-- OSC_CLK_in = 50 MHz  from the DE10-Lite oscillator
-- To run the Counter_mod12 with two CLK frequencies:
-- CLK_20Hz_SQ
-- CLK_1Hz_SQ 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in		:	IN		STD_LOGIC;
			CD 				:	IN		STD_LOGIC;
			CE				:	IN		STD_LOGIC;
			CLK_20Hz_SQ	:	OUT	STD_LOGIC;
			CLK_1Hz_SQ	:	OUT	STD_LOGIC			
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_1250000	IS                 
		PORT(	
			CD,CLK,CE		: 	IN  std_logic;
			TC1250000		:	OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT Freq_Div_20	IS                 
		PORT(	
			CD,CLK,CE		: 	IN  std_logic;
			TC20			:	OUT std_logic
		);
	END COMPONENT;		
	
	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_40Hz		:	std_logic;
	SIGNAL Pulsed_2Hz		:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_1250000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> CE,
			TC1250000	=> Pulsed_40Hz					  
		);
	
	Chip2	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_40Hz,
			Q 		=> CLK_20Hz_SQ				  
		);
		
	Chip3	: 	Freq_Div_20
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> Pulsed_40Hz,
			TC20			=> Pulsed_2Hz					  
		);	

	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2Hz,
			Q 		=> CLK_1Hz_SQ				  
		);		
		
	
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
